@@ -5,7 +5,7 @@
 library(RKorAPClient)
 library(ggplot2)
 
-freqPerDomain <- function(query, con = new("KorAPConnection", verbose = TRUE)) {
+freqPerDomain <- function(query, con = KorAPConnection(verbose = TRUE)) {
   g <- corpusQuery(con, query = query, vc="") %>%
     fetchAll() %>%
     slot("collectedMatches") %>%
@@ -16,7 +16,7 @@ freqPerDomain <- function(query, con = new("KorAPConnection", verbose = TRUE)) {
     mutate(total = (corpusStats(con, sprintf("textClass = /%s.*/", .$Domain)))$tokens) %>%
     ci(x = count) %>%
     ipm() %>%
-    { df <<- . } %>%
+    { df <- . } %>%
     ggplot(aes(x = Domain, y = ipm, ymin = conf.low, ymax = conf.high)) +
     geom_col() +
     geom_errorbar(width = .3, alpha = .3) +
